@@ -35,20 +35,24 @@ from calibre.utils.config import prefs, tweaks
 from calibre.utils.date import now as nowf, utcnow, UNDEFINED_DATE
 from calibre.utils.icu import sort_key
 
+
 def api(f):
     f.is_cache_api = True
     return f
 
+
 def read_api(f):
     f = api(f)
     f.is_read_api = True
     return f
 
+
 def write_api(f):
     f = api(f)
     f.is_read_api = False
     return f
 
+
 def wrap_simple(lock, func):
     @wraps(func)
     def call_func_with_lock(*args, **kwargs):
@@ -62,6 +66,7 @@ def wrap_simple(lock, func):
             return func(*args, **kwargs)
     return call_func_with_lock
 
+
 def run_import_plugins(path_or_stream, fmt):
     fmt = fmt.lower()
     if hasattr(path_or_stream, 'seek'):
@@ -74,6 +79,7 @@ def run_import_plugins(path_or_stream, fmt):
         path = path_or_stream
     return run_plugins_on_import(path, fmt)
 
+
 def _add_newbook_tag(mi):
     tags = prefs['new_book_tags']
     if tags:
@@ -84,8 +90,10 @@ def _add_newbook_tag(mi):
                 elif tag not in mi.tags:
                     mi.tags.append(tag)
 
+
 dynamic_category_preferences = frozenset({'grouped_search_make_user_categories', 'grouped_search_terms', 'user_categories'})
 
+
 class Cache(object):
 
     '''
@@ -131,6 +139,10 @@ class Cache(object):
     def new_api(self):
         return self
 
+    @property
+    def library_id(self):
+        return self.backend.library_id
+
     @property
     def safe_read_lock(self):
         ''' A safe read lock is a lock that does nothing if the thread already
@@ -800,6 +812,7 @@ class Cache(object):
                 path = self._field_for('path', book_id).replace('/', os.sep)
             except:
                 return ()
+
             def verify(fmt):
                 try:
                     name = self.fields['formats'].format_fname(book_id, fmt)
@@ -914,6 +927,7 @@ class Cache(object):
                     return virtual_fields[fm.get(field, field)].sort_keys_for_books(get_metadata, lang_map)
             if is_series:
                 idx_func = self.fields[idx].sort_keys_for_books(get_metadata, lang_map)
+
                 def skf(book_id):
                     return (func(book_id), idx_func(book_id))
                 return skf
@@ -1324,7 +1338,7 @@ class Cache(object):
         path = self._field_for('path', book_id)
         if path is None:
             # Theoretically, this should never happen, but apparently it
-            # does: http://www.mobileread.com/forums/showthread.php?t=233353
+            # does: https://www.mobileread.com/forums/showthread.php?t=233353
             self._update_path({book_id}, mark_as_dirtied=False)
             path = self._field_for('path', book_id)
 
@@ -1444,15 +1458,15 @@ class Cache(object):
         return _get_next_series_num_for_list(tuple(series_indices), unwrap=False)
 
     @read_api
-    def author_sort_from_authors(self, authors):
+    def author_sort_from_authors(self, authors, key_func=icu_lower):
         '''Given a list of authors, return the author_sort string for the authors,
         preferring the author sort associated with the author over the computed
         string. '''
         table = self.fields['authors'].table
         result = []
-        rmap = {icu_lower(v):k for k, v in table.id_map.iteritems()}
+        rmap = {key_func(v):k for k, v in table.id_map.iteritems()}
         for aut in authors:
-            aid = rmap.get(icu_lower(aut), None)
+            aid = rmap.get(key_func(aut), None)
             result.append(author_to_author_sort(aut) if aid is None else table.asort_map[aid])
         return ' & '.join(filter(None, result))
 
@@ -1476,7 +1490,7 @@ class Cache(object):
         if title:
             if isbytestring(title):
                 title = title.decode(preferred_encoding, 'replace')
-            q = icu_lower(title)
+            q = icu_lower(title).strip()
             for title in self.fields['title'].table.book_col_map.itervalues():
                 if q == icu_lower(title):
                     return True
@@ -2009,6 +2023,13 @@ class Cache(object):
 
     @write_api
     def close(self):
+        from calibre.customize.ui import available_library_closed_plugins
+        for plugin in available_library_closed_plugins():
+            try:
+                plugin.run(self)
+            except Exception:
+                import traceback
+                traceback.print_exc()
         self.backend.close()
 
     @write_api
@@ -2149,6 +2170,7 @@ class Cache(object):
         if progress is not None:
             progress(_('Completed'), total, total)
 
+
 def import_library(library_key, importer, library_path, progress=None, abort=None):
     from calibre.db.backend import DB
     metadata = importer.metadata[library_key]
