@@ -19,9 +19,11 @@ from calibre.ebooks.metadata.sources.base import (Source, Option, fixcase,
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.utils.localization import canonicalize_lang
 
+
 class CaptchaError(Exception):
     pass
 
+
 def parse_details_page(url, log, timeout, browser, domain):
     from calibre.utils.cleantext import clean_ascii_chars
     from calibre.ebooks.chardet import xml_to_unicode
@@ -62,7 +64,7 @@ def parse_details_page(url, log, timeout, browser, domain):
     if domain == 'jp':
         for a in root.xpath('//a[@href]'):
             if 'black-curtain-redirect.html' in a.get('href'):
-                url = 'http://amazon.co.jp'+a.get('href')
+                url = 'https://amazon.co.jp'+a.get('href')
                 log('Black curtain redirect found, following')
                 return parse_details_page(url, log, timeout, browser, domain)
 
@@ -77,6 +79,7 @@ def parse_details_page(url, log, timeout, browser, domain):
     selector = Select(root)
     return oraw, root, selector
 
+
 def parse_asin(root, log, url):
     try:
         link = root.xpath('//link[@rel="canonical" and @href]')
@@ -205,6 +208,7 @@ class Worker(Thread):  # Get details {{{
                  text()="Detalles del producto" or \
                  text()="Detalhes do produto" or \
                  text()="Productgegevens" or \
+                 text()="基本信息" or \
                  starts-with(text(), "登録情報")]/../div[@class="content"]
             '''
         # Editor: is for Spanish
@@ -229,9 +233,10 @@ class Worker(Thread):  # Get details {{{
                 or text() = "Idioma:" \
                 or starts-with(text(), "Langue") \
                 or starts-with(text(), "言語") \
+                or starts-with(text(), "语种")
                 ]
             '''
-        self.language_names = {'Language', 'Sprache', 'Lingua', 'Idioma', 'Langue', '言語', 'Taal'}
+        self.language_names = {'Language', 'Sprache', 'Lingua', 'Idioma', 'Langue', '言語', 'Taal', '语种'}
 
         self.tags_xpath = '''
             descendant::h2[
@@ -247,6 +252,7 @@ class Worker(Thread):  # Get details {{{
 
         self.ratings_pat = re.compile(
             r'([0-9.]+) ?(out of|von|van|su|étoiles sur|つ星のうち|de un máximo de|de) ([\d\.]+)( (stars|Sternen|stelle|estrellas|estrelas|sterren)){0,1}')
+        self.ratings_pat_cn = re.compile('平均([0-9.]+)')
 
         lm = {
                 'eng': ('English', 'Englisch', 'Engels'),
@@ -257,6 +263,7 @@ class Worker(Thread):  # Get details {{{
                 'jpn': ('Japanese', u'日本語'),
                 'por': ('Portuguese', 'Português'),
                 'nld': ('Dutch', 'Nederlands',),
+                'chs': ('Chinese', u'中文', u'简体中文'),
                 }
         self.lang_map = {}
         for code, names in lm.iteritems():
@@ -274,6 +281,8 @@ class Worker(Thread):  # Get details {{{
                 ''', re.X)
 
     def delocalize_datestr(self, raw):
+        if self.domain == 'cn':
+            return raw.replace('年','-').replace('月','-').replace('日','')
         if not self.months:
             return raw
         ans = raw.lower()
@@ -477,9 +486,14 @@ class Worker(Thread):  # Get details {{{
         if ratings:
             for elem in ratings[0].xpath('descendant::*[@title]'):
                 t = elem.get('title').strip()
-                m = self.ratings_pat.match(t)
-                if m is not None:
-                    return float(m.group(1))/float(m.group(3)) * 5
+                if self.domain == 'cn':
+                    m = self.ratings_pat_cn.match(t)
+                    if m is not None:
+                        return float(m.group(1))
+                else:
+                    m = self.ratings_pat.match(t)
+                    if m is not None:
+                        return float(m.group(1))/float(m.group(3)) * 5
 
     def _render_comments(self, desc):
         from calibre.library.comments import sanitize_comments_html
@@ -577,6 +591,19 @@ class Worker(Thread):  # Get details {{{
                         series = self.tostring(a[0], encoding=unicode, method='text', with_tail=False).strip()
                         if series:
                             ans = (series, series_index)
+        # This is found on newer Kindle edition pages on amazon.com
+        if ans == (None, None):
+            for b in root.xpath('//div[@id="reviewFeatureGroup"]/span/b'):
+                text = (b.text or '').strip()
+                m = re.match('Book\s+([0-9.]+)', text)
+                if m is not None:
+                    series_index = float(m.group(1))
+                    a = b.getparent().xpath('./a[@href]')
+                    if a:
+                        series = self.tostring(a[0], encoding=unicode, method='text', with_tail=False).partition('(')[0].strip()
+                        if series:
+                            ans = series, series_index
+
         if ans == (None, None):
             desc = root.xpath('//div[@id="ps-content"]/div[@class="buying"]')
             if desc:
@@ -669,7 +696,7 @@ class Worker(Thread):  # Get details {{{
             if 'data:' in src:
                 continue
             if 'loading-' in src:
-                js_img = re.search(br'"largeImage":"(http://[^"]+)",',raw)
+                js_img = re.search(br'"largeImage":"(https?://[^"]+)",',raw)
                 if js_img:
                     src = js_img.group(1).decode('utf-8')
             if ('/no-image-avail' not in src and 'loading-' not in src and '/no-img-sm' not in src):
@@ -748,6 +775,7 @@ class Worker(Thread):  # Get details {{{
                     return ans
 # }}}
 
+
 class Amazon(Source):
 
     name = 'Amazon.com'
@@ -771,6 +799,7 @@ class Amazon(Source):
             'es': _('Spain'),
             'br': _('Brazil'),
             'nl': _('Netherlands'),
+            'cn': _('China'),
     }
 
     options = (
@@ -816,29 +845,31 @@ class Amazon(Source):
                 x.startswith('identifier:amazon')] + [ident_name]
         self.touched_fields = frozenset(tf)
 
-    def get_domain_and_asin(self, identifiers):
+    def get_domain_and_asin(self, identifiers, extra_domains=()):
         for key, val in identifiers.iteritems():
             key = key.lower()
             if key in ('amazon', 'asin'):
                 return 'com', val
             if key.startswith('amazon_'):
-                domain = key.split('_')[-1]
-                if domain and domain in self.AMAZON_DOMAINS:
+                domain = key.partition('_')[-1]
+                if domain and (domain in self.AMAZON_DOMAINS or domain in extra_domains):
                     return domain, val
         return None, None
 
     def _get_book_url(self, identifiers):  # {{{
-        domain, asin = self.get_domain_and_asin(identifiers)
+        domain, asin = self.get_domain_and_asin(identifiers, extra_domains=('in', 'au', 'ca'))
         if domain and asin:
             url = None
             if domain == 'com':
-                url = 'http://amzn.com/'+asin
+                url = 'https://amzn.com/'+asin
             elif domain == 'uk':
-                url = 'http://www.amazon.co.uk/dp/'+asin
+                url = 'https://www.amazon.co.uk/dp/'+asin
             elif domain == 'br':
-                url = 'http://www.amazon.com.br/dp/'+asin
+                url = 'https://www.amazon.com.br/dp/'+asin
+            elif domain == 'au':
+                url = 'https://www.amazon.com.au/dp/' + asin
             else:
-                url = 'http://www.amazon.%s/dp/%s'%(domain, asin)
+                url = 'https://www.amazon.%s/dp/%s'%(domain, asin)
             if url:
                 idtype = 'amazon' if domain == 'com' else 'amazon_'+domain
                 return domain, idtype, asin, url
@@ -957,14 +988,14 @@ class Amazon(Source):
 
         if domain == 'jp':
             encode_to = 'Shift_JIS'
-        elif domain == 'nl':
+        elif domain == 'nl' or domain == 'cn':
             encode_to='utf-8'
         else:
             encode_to = 'latin1'
         encoded_q = dict([(x.encode(encode_to, 'ignore'), y.encode(encode_to,
             'ignore')) for x, y in
             q.iteritems()])
-        url = 'http://www.amazon.%s/s/?'%self.get_website_domain(domain) + urlencode(encoded_q)
+        url = 'https://www.amazon.%s/s/?'%self.get_website_domain(domain) + urlencode(encoded_q)
         return url, domain
 
     # }}}
@@ -1005,7 +1036,7 @@ class Amazon(Source):
             if title_ok(title):
                 url = a.get('href')
                 if url.startswith('/'):
-                    url = 'http://www.amazon.%s%s' % (self.get_website_domain(domain), url)
+                    url = 'https://www.amazon.%s%s' % (self.get_website_domain(domain), url)
                 matches.append(url)
 
         if not matches:
@@ -1020,7 +1051,7 @@ class Amazon(Source):
                     if title_ok(title):
                         url = a.get('href')
                         if url.startswith('/'):
-                            url = 'http://www.amazon.%s%s' % (self.get_website_domain(domain), url)
+                            url = 'https://www.amazon.%s%s' % (self.get_website_domain(domain), url)
                         matches.append(url)
                     break
 
@@ -1034,7 +1065,7 @@ class Amazon(Source):
                     if title_ok(title):
                         url = a.get('href')
                         if url.startswith('/'):
-                            url = 'http://www.amazon.%s%s' % (self.get_website_domain(domain), url)
+                            url = 'https://www.amazon.%s%s' % (self.get_website_domain(domain), url)
                         matches.append(url)
                     break
         if not matches and root.xpath('//form[@action="/errors/validateCaptcha"]'):
@@ -1201,6 +1232,7 @@ class Amazon(Source):
             log.exception('Failed to download cover from:', cached_url)
     # }}}
 
+
 if __name__ == '__main__':  # tests {{{
     # To run these test use: calibre-debug src/calibre/ebooks/metadata/sources/amazon.py
     from calibre.ebooks.metadata.sources.test import (test_identify_plugin,
@@ -1362,6 +1394,20 @@ if __name__ == '__main__':  # tests {{{
             ),
     ]  # }}}
 
+    cn_tests = [  # {{{
+            (
+                {'identifiers':{'isbn':'9787115369512'}},
+                [title_test('若为自由故 自由软件之父理查德斯托曼传', exact=True), authors_test(['[美]sam Williams', '邓楠，李凡希'])]
+            ),
+            (
+                {'title':'爱上Raspberry Pi'},
+                [title_test('爱上Raspberry Pi',
+                    exact=True), authors_test(['Matt Richardson', 'Shawn Wallace', '李凡希'])
+                 ]
+
+            ),
+    ]  # }}}
+
     def do_test(domain, start=0, stop=None):
         tests = globals().get(domain+'_tests')
         if stop is None:
