@@ -8,29 +8,29 @@ import os, subprocess, errno, shutil, tempfile, sys
 from io import BytesIO
 from threading import Thread
 
-from PyQt5.Qt import QImage, QByteArray, QBuffer, Qt, QImageReader, QColor, QImageWriter, QTransform
+from PyQt5.Qt import QImage, QByteArray, QBuffer, Qt, QImageReader, QColor, QImageWriter, QTransform, QPixmap
 from calibre import fit_image, force_unicode
-from calibre.constants import iswindows, plugins, get_version
+from calibre.constants import iswindows, plugins
 from calibre.utils.config_base import tweaks
 from calibre.utils.filenames import atomic_rename
 
 # Utilities {{{
 imageops, imageops_err = plugins['imageops']
 if imageops is None:
-    if '*' in get_version():
-        raise RuntimeError('You are running from source, which requires the new binary module, imageops. You can'
-                           ' get it by installing the betas from: http://www.mobileread.com/forums/showthread.php?t=274030')
     raise RuntimeError(imageops_err)
 
+
 class NotImage(ValueError):
     pass
 
+
 def normalize_format_name(fmt):
     fmt = fmt.lower()
     if fmt == 'jpg':
         fmt = 'jpeg'
     return fmt
 
+
 def get_exe_path(name):
     from calibre.ebooks.pdf.pdftohtml import PDFTOHTML
     base = os.path.dirname(PDFTOHTML)
@@ -43,10 +43,12 @@ def get_exe_path(name):
 
 # Loading images {{{
 
+
 def null_image():
     ' Create an invalid image. For internal use. '
     return QImage()
 
+
 def image_from_data(data):
     ' Create an image object from data, which should be a bytestring. '
     if isinstance(data, QImage):
@@ -56,11 +58,13 @@ def image_from_data(data):
         raise NotImage('Not a valid image')
     return i
 
+
 def image_from_path(path):
     ' Load an image from the specified path. '
     with lopen(path, 'rb') as f:
         return image_from_data(f.read())
 
+
 def image_from_x(x):
     ' Create an image from a bytestring or a path or a file like object. '
     if isinstance(x, type('')):
@@ -71,10 +75,13 @@ def image_from_x(x):
         return image_from_data(x)
     if isinstance(x, bytearray):
         return image_from_data(bytes(x))
+    if isinstance(x, QPixmap):
+        return x.toImage()
     raise TypeError('Unknown image src type: %s' % type(x))
 
+
 def image_and_format_from_data(data):
-    ' Create an image object from the specified data which should be a bytsestring and also return the format of the image '
+    ' Create an image object from the specified data which should be a bytestring and also return the format of the image '
     ba = QByteArray(data)
     buf = QBuffer(ba)
     buf.open(QBuffer.ReadOnly)
@@ -85,6 +92,7 @@ def image_and_format_from_data(data):
 
 # Saving images {{{
 
+
 def image_to_data(img, compression_quality=95, fmt='JPEG', png_compression_level=9, jpeg_optimized=True, jpeg_progressive=False):
     '''
     Serialize image to bytestring in the specified format.
@@ -126,6 +134,7 @@ def image_to_data(img, compression_quality=95, fmt='JPEG', png_compression_level
         raise ValueError('Failed to export image as ' + fmt + ' with error: ' + w.errorString())
     return ba.data()
 
+
 def save_image(img, path, **kw):
     ''' Save image to the specified path. Image format is taken from the file
     extension. You can pass the same keyword arguments as for the
@@ -135,6 +144,7 @@ def save_image(img, path, **kw):
     with lopen(path, 'wb') as f:
         f.write(image_to_data(image_from_data(img), **kw))
 
+
 def save_cover_data_to(data, path=None, bgcolor='#ffffff', resize_to=None, compression_quality=90, minify_to=None, grayscale=False, data_fmt='jpeg'):
     '''
     Saves image in data to path, in the format specified by the path
@@ -186,6 +196,7 @@ def save_cover_data_to(data, path=None, bgcolor='#ffffff', resize_to=None, compr
 
 # Overlaying images {{{
 
+
 def blend_on_canvas(img, width, height, bgcolor='#ffffff'):
     ' Blend the `img` onto a canvas with the specified background color and size '
     w, h = img.width(), img.height()
@@ -198,6 +209,7 @@ def blend_on_canvas(img, width, height, bgcolor='#ffffff'):
     overlay_image(img, canvas, (width - w)//2, (height - h)//2)
     return canvas
 
+
 class Canvas(object):
 
     def __init__(self, width, height, bgcolor='#ffffff'):
@@ -217,6 +229,7 @@ class Canvas(object):
     def export(self, fmt='JPEG', compression_quality=95):
         return image_to_data(self.img, compression_quality=compression_quality, fmt=fmt)
 
+
 def create_canvas(width, height, bgcolor='#ffffff'):
     'Create a blank canvas of the specified size and color '
     img = QImage(width, height, QImage.Format_RGB32)
@@ -233,12 +246,14 @@ def overlay_image(img, canvas=None, left=0, top=0):
     imageops.overlay(img, canvas, left, top)
     return canvas
 
+
 def texture_image(canvas, texture):
     ' Repeatedly tile the image `texture` across and down the image `canvas` '
     if canvas.hasAlphaChannel():
         canvas = blend_image(canvas)
     return imageops.texture_image(canvas, texture)
 
+
 def blend_image(img, bgcolor='#ffffff'):
     ' Used to convert images that have semi-transparent pixels to opaque by blending with the specified color '
     canvas = QImage(img.size(), QImage.Format_RGB32)
@@ -249,6 +264,7 @@ def blend_image(img, bgcolor='#ffffff'):
 
 # Image borders {{{
 
+
 def add_borders_to_image(img, left=0, top=0, right=0, bottom=0, border_color='#ffffff'):
     img = image_from_data(img)
     if not (left > 0 or right > 0 or top > 0 or bottom > 0):
@@ -258,6 +274,7 @@ def add_borders_to_image(img, left=0, top=0, right=0, bottom=0, border_color='#f
     overlay_image(img, canvas, left, top)
     return canvas
 
+
 def remove_borders_from_image(img, fuzz=None):
     ''' Try to auto-detect and remove any borders from the image. Returns
     the image itself if no borders could be removed. `fuzz` is a measure of
@@ -271,9 +288,11 @@ def remove_borders_from_image(img, fuzz=None):
 
 # Cropping/scaling of images {{{
 
+
 def resize_image(img, width, height):
     return img.scaled(int(width), int(height), Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
 
+
 def resize_to_fit(img, width, height):
     img = image_from_data(img)
     resize_needed, nw, nh = fit_image(img.width(), img.height(), width, height)
@@ -281,11 +300,13 @@ def resize_to_fit(img, width, height):
         resize_image(img, nw, nh)
     return resize_needed, img
 
+
 def clone_image(img):
     ''' Returns a shallow copy of the image. However, the underlying data buffer
     will be automatically copied-on-write '''
     return QImage(img)
 
+
 def scale_image(data, width=60, height=80, compression_quality=70, as_png=False, preserve_aspect_ratio=True):
     ''' Scale an image, returning it as either JPEG or PNG data (bytestring).
     Transparency is alpha blended with white when converting to JPEG. Is thread
@@ -304,6 +325,7 @@ def scale_image(data, width=60, height=80, compression_quality=70, as_png=False,
     w, h = img.width(), img.height()
     return w, h, image_to_data(img, compression_quality=compression_quality, fmt=fmt)
 
+
 def crop_image(img, x, y, width, height):
     '''
     Return the specified section of the image.
@@ -322,18 +344,22 @@ def crop_image(img, x, y, width, height):
 
 # Image transformations {{{
 
+
 def grayscale_image(img):
     return imageops.grayscale(image_from_data(img))
 
+
 def set_image_opacity(img, alpha=0.5):
     ''' Change the opacity of `img`. Note that the alpha value is multiplied to
     any existing alpha values, so you cannot use this function to convert a
     semi-transparent image to an opaque one. For that use `blend_image()`. '''
     return imageops.set_opacity(image_from_data(img), alpha)
 
+
 def flip_image(img, horizontal=False, vertical=False):
     return image_from_data(img).mirrored(horizontal, vertical)
 
+
 def image_has_transparent_pixels(img):
     ' Return True iff the image has at least one semi-transparent pixel '
     img = image_from_data(img)
@@ -341,26 +367,33 @@ def image_has_transparent_pixels(img):
         return False
     return imageops.has_transparent_pixels(img)
 
+
 def rotate_image(img, degrees):
     t = QTransform()
     t.rotate(degrees)
     return image_from_data(img).transformed(t)
 
+
 def gaussian_sharpen_image(img, radius=0, sigma=3, high_quality=True):
     return imageops.gaussian_sharpen(image_from_data(img), max(0, radius), sigma, high_quality)
 
+
 def gaussian_blur_image(img, radius=-1, sigma=3):
     return imageops.gaussian_blur(image_from_data(img), max(0, radius), sigma)
 
+
 def despeckle_image(img):
     return imageops.despeckle(image_from_data(img))
 
+
 def oil_paint_image(img, radius=-1, high_quality=True):
     return imageops.oil_paint(image_from_data(img), radius, high_quality)
 
+
 def normalize_image(img):
     return imageops.normalize(image_from_data(img))
 
+
 def quantize_image(img, max_colors=256, dither=True, palette=''):
     ''' Quantize the image to contain a maximum of `max_colors` colors. By
     default a palette is chosen automatically, if you want to use a fixed
@@ -383,6 +416,7 @@ def quantize_image(img, max_colors=256, dither=True, palette=''):
 
 # Optimization of images {{{
 
+
 def run_optimizer(file_path, cmd, as_filter=False, input_data=None):
     file_path = os.path.abspath(file_path)
     cwd = os.path.dirname(file_path)
@@ -396,6 +430,7 @@ def run_optimizer(file_path, cmd, as_filter=False, input_data=None):
         else:
             os.close(fd)
         iname, oname = os.path.basename(file_path), os.path.basename(outfile)
+
         def repl(q, r):
             cmd[cmd.index(q)] = r
         if not as_filter:
@@ -414,6 +449,7 @@ def run_optimizer(file_path, cmd, as_filter=False, input_data=None):
         stderr = p.stderr if as_filter else p.stdout
         if as_filter:
             src = input_data or open(file_path, 'rb')
+
             def copy(src, dest):
                 try:
                     shutil.copyfileobj(src, dest)
@@ -451,16 +487,19 @@ def run_optimizer(file_path, cmd, as_filter=False, input_data=None):
             if err.errno != errno.ENOENT:
                 raise
 
+
 def optimize_jpeg(file_path):
     exe = get_exe_path('jpegtran')
     cmd = [exe] + '-copy none -optimize -progressive -maxmemory 100M -outfile'.split() + [False, True]
     return run_optimizer(file_path, cmd)
 
+
 def optimize_png(file_path):
     exe = get_exe_path('optipng')
     cmd = [exe] + '-fix -clobber -strip all -o7 -out'.split() + [False, True]
     return run_optimizer(file_path, cmd)
 
+
 def encode_jpeg(file_path, quality=80):
     from calibre.utils.speedups import ReadOnlyFileBuffer
     quality = max(0, min(100, int(quality)))
@@ -477,12 +516,14 @@ def encode_jpeg(file_path, quality=80):
     return run_optimizer(file_path, cmd, as_filter=True, input_data=ReadOnlyFileBuffer(ba.data()))
 # }}}
 
+
 def test():  # {{{
     from calibre.ptempfile import TemporaryDirectory
     from calibre import CurrentDir
     from glob import glob
+    img = image_from_data(I('lt.png', data=True, allow_user_override=False))
     with TemporaryDirectory() as tdir, CurrentDir(tdir):
-        shutil.copyfile(I('devices/kindle.jpg', allow_user_override=False), 'test.jpg')
+        save_image(img, 'test.jpg')
         ret = optimize_jpeg('test.jpg')
         if ret is not None:
             raise SystemExit('optimize_jpeg failed: %s' % ret)
@@ -495,7 +536,6 @@ def test():  # {{{
             raise SystemExit('optimize_png failed: %s' % ret)
         if glob('*.bak'):
             raise SystemExit('Spurious .bak files left behind')
-    img = image_from_data(I('devices/kindle.jpg', data=True, allow_user_override=False))
     quantize_image(img)
     oil_paint_image(img)
     gaussian_sharpen_image(img)
@@ -505,6 +545,7 @@ def test():  # {{{
     image_to_data(img, fmt='GIF')
 # }}}
 
+
 if __name__ == '__main__':  # {{{
     args = sys.argv[1:]
     infile = args.pop(0)
